#!/usr/bin/perl
#
# $Id: perlpat.pl,v 1.7 2001/04/21 16:42:52 erik Exp $
#
# (C) 2000-2001 Erik Oliver
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# User Tunable Settings...
## Note: WORKING_DIR should have mode +t, e.g. /tmp
my $WORKING_DIR = "/home/httpd/cgi-bin/tmp";
my $DEBUG       = 1; # 0 for no debugging, 1 for debugging
my $SAVE_HTML	= 1; # 0 for no full text, 1 for full text
my $FETCHCMD1    = "/home/httpd/cgi-bin/retrieve.pl";
my $FETCHCMD2    = "/home/httpd/cgi-bin/pregrantretrieve.pl";

# request required packages
require strict;
require CGI;
require Sys::Hostname;
require MIME::Lite;

use strict;
use CGI qw/:standard/;
use Sys::Hostname;
use MIME::Lite;

# Global Variables
my $em = "";
my $pn="";
my $su="";
my $fr="";
my $body="";
my $clr="";

# no buffering for I/O
$| = 1;

# pre-declare my subroutines

# footprints = file named pat{#######}.fetch containing PID of
# process trying to obtain the patent
sub clear_footprint; # clear out footprint
sub check_footprint($); # look to see if a .fetch file has been placed
						# for pat # of arg1
sub set_footprint;   # place a footprint if safe

# err_msg should be used instead of die as it e-mails the error
# for obvious reasons it should not be used until $em (the to
# e-mail address has been parsed successfully
sub err_msg;

# these are the functions: fetch_mail_pat does the heavy
# lifting in the child process
sub fetch_mail_pat;
# sends a mail message once a .pdf file is in the appopriate place
sub mail_pat;
# invokes another program to get the patent
sub get_pat;
# logs the request in a log file
sub log_pat;

# "main" - displays form or handles processing of same
if (!param()) {
	# first call...
	print header,
		start_html('PatentMailer'),
		h1('PatentMailer'), start_form, hr,
		"E-mail address: ",textfield('em'),br,
		"Patent Number: ",textfield('pn'),br,
		submit, hr,
		h2('Optional Fields'),
		"<UL>",
		"<LI>Subject Line: ",textfield('su'),br,
		"<LI>From E-mail: ",textfield('fr'),br,
		"<LI>Body: ",textarea(-name=>'msg',-rows=>5,-columns=>30),br,
		"<LI>Clear cached copy? ",checkbox(-name=>'clr',-label=>''),
		"</UL>",
		submit,end_form;
	print hr,
		"Note: Retrieve design patent with 'D<6 digit #>', retrieve ",
		"pregrant publications with 11-digit application number.<BR>",
		end_html;
} else {
	my $errct = 0;
	my $host = hostname();
	my $pid;

	# with submit data in param('<fieldname>')
	print header,
		start_html('Results');

	# check referer
	#print "\nreferer>",referer(),"<referer\n";

	# we just accept e-mail addresses as long as they are non-null
	$em = param('em');
	if ( ! $em ) { $errct++; print p,"You must provide an e-mail address.\n"; }

	# append our hostname for completeness
	if ($em !~ m|@|) {
		$em = "${em}@" . ${host};
	}

	# check for reasonably valid patent number and 
	# strip punctuation type things
	$pn = param('pn');
	#$pn =~ s/\D//g;
	# remove commas and slashes by removing non-alphanumerics
	$pn =~ s/\W//g; 
	$pn =~ s/^d/D/; # make a d for design a caps

	if ( $pn !~ m/^\d{7}$/ && 
		 $pn !~ m/^D\d{6}$/ && 
		 $pn !~ m/^\d{11}$/) { 
		$errct++; 
		print p, "Patent numbers have 7 digits (or should be prefixed by a \"D\"",
		"followed by the 6 digit design patent number); pre grant
publications have an 11 digit application number. you entered \"",
			param('pn'),"\"\n"; 
	}

	# assign a good default subject if none was provided by user...
	$su = param('su');
	if ( ! $su ) { $su = "United States Patent No. ${pn}"; }

	# assign a good default from if none was provided by user
	$fr = param('fr');
	if ( ! $fr ) { $fr = "nobody@" . ${host}; }
	if ($fr !~ m|@|) {
		$fr = "${fr}@" . ${host};
	}

	# assign clr to either y or n from "on" for checked
	if ( param('clr') eq "on" ) {
		$clr="y";
	} else {
		$clr="n";
	}

	# supply a reasonable default body...
	if(! param('msg') ) {
		$body = "US Pat No ${pn} is attached.\n";
	} else {
		$body = param('msg');
	};

	# check for other retrievals of same patent
	my $pid = check_footprint($pn);
    if ( $pid ) {
		$errct++;
		print p,"Sorry, only one request for a patent is allowed at ",
				"a time. It looks like PID=($pid) is still fetching ",
				"the patent.\n";
    }

	# Decision point if errct > 0, then halt... 
	# otherwise invoke mail/fetch process
	if ( $errct > 0 ) {
		print p,"<B>Please make the required corrections by ",
				"using \"Back\" and \"Submit\". Or, wait a few ",
				" minutes if the patent is already being requested.</B>\n";
		print br;
		print end_html,"\n\n";
	} else {
		my $logfile = $WORKING_DIR . "/pat" . $pn . ".log";
		# fork off a process to go do the hard stuff
		# this also allows the web server to close the connection
		print h1("Patent Request Received"),p;
		print qq|
<P>All further information, including the PDF, will be sent to you at
&lt;mailto:${em}&gt;. If there are any errors, the e-mail will include as
much information as possible to help you understand the problem.

<P><EM>Note:</EM> Many 'errors' that are returned are the result of
tempermentalness on the part of the USPTO web site. When possible, the URL
at the USPTO site that is applicable will be shown to you so that you can
try and access the page yourself.<BR>

<P>Also, do not submit multiple requests for the same patent simultaneously,
be patient, the process can take several minutes. If you do place a a
second request, it will return an error message unless the original
process has died for some reason.

<BR>
|;
			
		print end_html,"\n\n";
		
		FORK: {
			if ($pid = fork) {
				# parent, terminate output
				close STDOUT;
				close STDIN;
				close STDERR;
			} elsif (defined $pid) {
				# child process
				close STDOUT;
				close STDIN;
				close STDERR;
				
				# clear older logs	
				unlink("$logfile") unless (! -f "$logfile");
				open(STDOUT,">> $logfile");
				open(STDERR,">> $logfile");
				print "Logging information for request for ${pn} ";

				fetch_mail_pat();

				close STDOUT;
				close STDERR;
				unlink("$logfile") unless (! -f "$logfile");
				
			} elsif ( $! =~ /No more process/) {
				# try sleeping for a bit
				sleep 5;
				redo FORK;
			} else {
				die "Can't fork: $!\n";
			}
		}
	}
}

sub clear_footprint {
	my $pidfile = $WORKING_DIR . "/pat" . $pn . ".fetch";
	unlink ($pidfile) unless (! -f $pidfile);

	# and any temporary TIFF files
	chdir $WORKING_DIR;
	#system("/bin/rm $WORKING_DIR/pat${pn}.tiff.*");
	unlink(<$WORKING_DIR/pat${pn}.tiff.*>);
}

sub log_pat {
	my $logfile = $WORKING_DIR . "/patentmailer.log";
	open(FH,">> $logfile");
	print FH localtime() . " $pn $em\n";
	close(FH);
}

sub err_msg {
	my $msg = MIME::Lite->new(
		From => $fr,
		To   => $em,
		Subject => "[PatentMailer] Error while retrieving ${pn}",
		Type    => 'multipart/mixed'
	);
	my $logfile = $WORKING_DIR . "/pat" . $pn . ".log";

	$_ = shift @_;

	# e-mail the user the basic error message in first arg
	$msg->attach(
		Type => 'TEXT',
		Data => "$_");

	# if debugging is selected and there is a logfile, send the log	
	if($DEBUG && -f $logfile) {
		$msg->attach(
			Type     => 'TEXT',
			Path     => $logfile,
			Filename => 'log.txt'
		); 
	}
	
	# actually cause the message to be sent
	$msg->send;

	# check before removing the footprint
	$_ = shift @_;
	clear_footprint() unless ( m/no/);
	exit 1;
}

sub get_pat {
	# retrieve a patent and store a PDF
	my $logfile = $WORKING_DIR . "/pat" . $pn . ".log";

	$_ = shift @_;
	chdir($WORKING_DIR);
	if (m/^\d{11}$/) {
		# pregrant pubs handled separately
		system("$FETCHCMD2 $_");
	} else {
		# issued patents (incl. design patents)
		system("$FETCHCMD1 $_");
	}
}

sub mail_pat {
	my $msg = MIME::Lite->new(
		From => $fr,
		To   => $em,
		Subject => "[PatentMailer] ${su}",
		Type    => 'multipart/mixed'
	);
	my $logfile = $WORKING_DIR . "/pat" . $pn . ".log";
	my $patfile = $WORKING_DIR . "/pat" . $pn . ".pdf";
	my $htmlfile = $WORKING_DIR . "/pat" . $pn . ".html";

	# first the body of the message, e.g. $msg
	$msg->attach(
		Type		=> 'TEXT',
		Data		=> "$body"
	);

	# now attach the patent itself
	$msg->attach(
		Type		=> 'Application/PDF',
		Path		=> $patfile,
		Filename	=> "pat${pn}.pdf"
	);

	# if SAVE_HTML request and available, send full text
	if($SAVE_HTML && -f $htmlfile) {
		$msg->attach(
			Type 		=> 'text/html',
			Path		=> $htmlfile,
			Filename	=> "pat${pn}.html"
		);
	}
	
	#lastly, if debugging and there is a log file, send it
	if($DEBUG && -f $logfile) {
		$msg->attach(
			Type     => 'TEXT',
			Path     => $logfile,
			Filename => 'log.txt'
		); 
	}

	# and send it
	$msg->send;
}

sub check_footprint ($) {
	# returns PID of running process or 0 for no running processes
	$_ = shift @_;

	my $pidfile = $WORKING_DIR . "/pat" . $_ . ".fetch";
	my $cnt;
	my $pid = `cat $pidfile`;

	if ( -f $pidfile ) {
		# check if the process designated is actually running
		$cnt = kill 0, $pid;
		return ($cnt ? $pid : 0);
	} else {
		return 0;
	}
}

sub set_footprint {
	my $pidfile = $WORKING_DIR . "/pat" . $pn . ".fetch";

	# first things first place a "footprint" 
	if ( check_footprint ($pn) ) {
		err_msg("Sorry, another process is already attempting retrieval.",
				"no");
	}

	clear_footprint();
	# place our "footprint"
	open(PIDFILE,"> $pidfile") || err_msg("Can't open $pidfile: $!\n");
	print PIDFILE "$$";
	close PIDFILE;
}
	
sub fetch_mail_pat {
	my $outfile = $WORKING_DIR . "/pat" . $pn;

	# record the request in the logfile
	log_pat();

	# set a "footprint", e.g a file with our PID
	# to prevent multiple requests for the same patent
	# concurrently
	set_footprint(); # will die on mult simul req for same pat

	# clear cached copy if requested by user
	if ($clr eq "y" && -f "pat${pn}.pdf") { unlink("pat${pn}.pdf") };

	# request patent unless cached
	get_pat($pn) unless(-f "pat${pn}.pdf");

	# ok now to mail the patent to you together with any log output
	if ( -f "pat${pn}.pdf" ) {
		mail_pat();
	} else {
		err_msg("Patent file not found after call to get_pat.");
	}

	# clean up our footprint
	clear_footprint();
}
