#!/usr/bin/perl -w
#
# $Id: retrieve.pl,v 1.6 2003/02/02 18:48:56 erik Exp $
#
# (C) 2000-2001, 2003 Erik Oliver
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# constants
my $WORKING_DIR = "/Users/erik/tmp/pmailer";
my $TIFF2PS = "/sw/bin/tiff2ps";
my $PS2PDF  = "/sw/bin/ps2pdf";
my $DEBUG   = 1; #0 for no debugging

require LWP::UserAgent;
require HTTP::Request;

use strict;
use LWP::UserAgent;

sub herr {
	$_ = shift @_;
	print;
	print "\n";
	exit 1;
}


sub get_page($) {
	$_ = shift @_;

	my $ua = new LWP::UserAgent;
	$ua->agent("Mozilla/5.001 (Gecko like)");
	$ua->timeout(60);

	my $req = HTTP::Request->new(GET => $_ );
	my $res = $ua->request($req);


	if ($res->is_success) {
		return $res->content;
	} else {
		my $tmp = $res->status_line;
		herr("HTTP Error ($tmp) retrieving:\n <$_>\n");
	}
}

sub cleartiffs($) {
	my $pn = shift @_;
	
	# we call system rm
	system("/bin/rm $WORKING_DIR/pat${pn}.tiff.* 2>/dev/null");
}

sub get_pat($) {
	my $loop ="";
	my $errct=0 ;
	my $pn = "";
	my $result = "";
	my $nexturi = "";
	my $design = 0;
	my $pregrantpub = 0;
	my $imgserver = "";
	my $idkey = "&IDKey=NONE";

	my $usagemsg = qq@
usage: $0 <seven digit utility patent number>         |
       $0 D<six digit design patent number>           |
       $0 YYYY<7 digit publication number> 
@;


	foreach $loop ($TIFF2PS,$PS2PDF)
	{
		if ( ! -x $loop) {
			$errct++;
			print "The following is required: $loop\n";
		}
	}
	if ($errct > 0 ) {
			print qq|
  Most of the command line tools required by this program can be obtained
  by installing "ghostscript" and "libtiff" on your machine.

  Once installed, you should update the variables \$TIFF2PS and 
  \$PS2PDF at the top of this file appropriately.

  Ghostscript can be found at <http://www.cs.wisc.edu/~ghost/>.
  LibTiff can be found at <http://www.libtiff.org/>.

  Note: Developed with Ghostscript 6.0 and Libtiff 3.5.5.

|;
		herr("Exiting...");
 	} 

	$_ = shift @_;

	if ( ! $_ ) {
		herr("$usagemsg");
	}

	s/\W//g; # strip non-alphanumeric
	s/^d/D/; # make a d for design a caps

	if ( ! $_ || ! (m/^\d{7}$/ || m/^D\d{6}$/  || m/^\d{11}$/ ) ) {
		herr("$usagemsg");
	}

	# for design patents, put the digits (only) into $design
	if ( m/^D/ ) { $design = $_; $design =~ s/D//; }
	if ( m/\d{11}/) { $pregrantpub = 1; }
	
	$pn = $_;


	# unlink any tiffs, html, or pdf
	cleartiffs($pn);
	unlink(<$WORKING_DIR/pat${pn}.pdf>);

	# select the appropriate server

	if ( $pregrantpub ) {
		$imgserver = "http://aiwX.uspto.gov";
	} else {
		# utility + design
		$imgserver = "http://patimgX.uspto.gov";
	}

	# extract last two digits of $pn to compute img server 1 or 2
	$pn =~ m/(\d\d)$/;
	if ($1 < 50 ) { $imgserver =~ s/X/1/; } else { $imgserver =~ s/X/2/; }

	# lets get the first image
	# and figure out the # of pages
	if ( $design ) {
		$nexturi = $imgserver . "/.piw?Docid=D0${design}&PageNum=1" . $idkey;
	} elsif ( $pregrantpub ) {
		$nexturi = $imgserver . "/.aiw?Docid=us${pn}ki&PageNum=1" . $idkey;
	} else {	
		$nexturi = $imgserver . "/.piw?Docid=US00${pn}&PageNum=1" . $idkey;
	}

	print "Base URL = <$nexturi>\n" if ($DEBUG);

	$result = get_page($nexturi);

	# figure out how many pages.
	print "Determining number of pages...\n" if ($DEBUG);

	my $numpages = 0;
	my $tiffstream = $WORKING_DIR . "/pat" . ${pn} . ".tiff";
	my $tifffile = "";

	$result =~ m|NumPages=(\d+)| ||
		herr("Could not determine number of pages.\n URL = <$nexturi>\n$result");

    	$numpages = $1;

	print "...got it, $numpages.\n" if ($DEBUG);

	# find true Idkey
	$result =~ m|(IDKey=[A-Z0-9]*)| ||
		herr("Could not find IDKey.\n URL = <$nexturi>\n$result");

	$idkey = "&" . $1;

	chdir($WORKING_DIR);
	foreach $loop (1 .. $numpages) {
		print "[" if ($DEBUG);
		if ( $design ) {
			# handle design patent retrieval
			$nexturi = $imgserver . "/.DImg?Docid=D0${design}&PageNum=$loop" .
					$idkey . "&ImageFormat=tif";
		} elsif ( $pregrantpub ) {
			$nexturi = $imgserver . "/.DImg?Docid=us${pn}ki&PageNum=$loop" . $idkey . "&ImageFormat=tif";
		} else {
			# utility patent retrieval
			$nexturi = $imgserver . "/.DImg?Docid=US00${pn}&PageNum=$loop" .
					$idkey . "&ImageFormat=tif";
		}
		$tifffile = sprintf("%s.%04d",$tiffstream,$loop);
		print "$loop" if ($DEBUG);
		open(TIFFHAND,"> $tifffile");
		print TIFFHAND get_page($nexturi);
		close(TIFFHAND);
		print "]\n" if ($DEBUG);
	}
	
	# ok we should have all the tiffs in our workingdir
	# make the pdf
	# note: here is a good hoook for other nefarious things you might
	#       want to do with the tiffs...

	print "Converting TIFFs into a PDF, this will take some time...\n" if ($DEBUG);

	system("$TIFF2PS ${tiffstream}.* 2>/dev/null | $PS2PDF - > pat${pn}.pdf");

	# clean up tiff files
	cleartiffs($pn);
}

get_pat($ARGV[0]);
